@@ -20,15 +20,17 @@
 # spell-checker:ignore dwim
 # pylint: disable=too-many-lines
 """Generic utility helpers."""
+
 from __future__ import annotations
 
 import ast
+import collections.abc
 import contextlib
 import inspect
 import logging
 import os
 import re
-from collections.abc import ItemsView, Iterator, Mapping, Sequence
+from collections.abc import ItemsView, Iterable, Iterator, Mapping, Sequence
 from dataclasses import _MISSING_TYPE, dataclass, field
 from functools import cache, lru_cache
 from pathlib import Path
@@ -37,10 +39,13 @@ from typing import TYPE_CHECKING, Any
 import ruamel.yaml.parser
 import yaml
 from ansible.errors import AnsibleError, AnsibleParserError
+from ansible.module_utils._text import to_bytes
 from ansible.module_utils.parsing.convert_bool import boolean
 from ansible.parsing.dataloader import DataLoader
 from ansible.parsing.mod_args import ModuleArgsParser
 from ansible.parsing.plugin_docs import read_docstring
+from ansible.parsing.splitter import split_args
+from ansible.parsing.vault import PromptVaultSecret
 from ansible.parsing.yaml.constructor import AnsibleConstructor, AnsibleMapping
 from ansible.parsing.yaml.loader import AnsibleLoader
 from ansible.parsing.yaml.objects import AnsibleBaseYAMLObject, AnsibleSequence
@@ -53,7 +58,9 @@ from ansible.plugins.loader import (
 from ansible.template import Templar
 from ansible.utils.collection_loader import AnsibleCollectionConfig
 from yaml.composer import Composer
+from yaml.parser import ParserError
 from yaml.representer import RepresenterError
+from yaml.scanner import ScannerError
 
 from ansiblelint._internal.rules import (
     AnsibleParserErrorRule,
@@ -67,7 +74,6 @@ from ansiblelint.constants import (
     INCLUSION_ACTION_NAMES,
     LINE_NUMBER_KEY,
     NESTED_TASK_KEYS,
-    PLAYBOOK_TASK_KEYWORDS,
     ROLE_IMPORT_ACTION_NAMES,
     SKIPPED_RULES_KEY,
     FileType,
@@ -75,7 +81,7 @@ from ansiblelint.constants import (
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, discover_lintables
 from ansiblelint.skip_utils import is_nested_task
-from ansiblelint.text import has_jinja, removeprefix
+from ansiblelint.text import has_jinja, is_fqcn, removeprefix
 
 if TYPE_CHECKING:
     from ansiblelint.rules import RulesCollection
@@ -83,7 +89,6 @@ if TYPE_CHECKING:
 # string as the password to enable such yaml files to be opened and parsed
 # successfully.
 DEFAULT_VAULT_PASSWORD = "x"  # noqa: S105
-COLLECTION_PLAY_RE = re.compile(r"^[\w\d_]+\.[\w\d_]+\.[\w\d_]+$")
 
 PLAYBOOK_DIR = os.environ.get("ANSIBLE_PLAYBOOK_DIR", None)
 
@@ -91,11 +96,14 @@ PLAYBOOK_DIR = os.environ.get("ANSIBLE_PLAYBOOK_DIR", None)
 _logger = logging.getLogger(__name__)
 
 
-def parse_yaml_from_file(filepath: str) -> AnsibleBaseYAMLObject:
+def parse_yaml_from_file(filepath: str) -> AnsibleBaseYAMLObject:  # type: ignore[no-any-unimported]
     """Extract a decrypted YAML object from file."""
     dataloader = DataLoader()
-    if hasattr(dataloader, "set_vault_password"):
-        dataloader.set_vault_password(DEFAULT_VAULT_PASSWORD)
+    if hasattr(dataloader, "set_vault_secrets"):
+        dataloader.set_vault_secrets(
+            [("default", PromptVaultSecret(_bytes=to_bytes(DEFAULT_VAULT_PASSWORD)))]
+        )
+
     return dataloader.load_from_file(filepath)
 
 
@@ -106,7 +114,7 @@ def path_dwim(basedir: str, given: str) -> str:
     return str(dataloader.path_dwim(given))
 
 
-def ansible_templar(basedir: Path, templatevars: Any) -> Templar:
+def ansible_templar(basedir: Path, templatevars: Any) -> Templar:  # type: ignore[no-any-unimported]
     """Create an Ansible Templar using templatevars."""
     # `basedir` is the directory containing the lintable file.
     # Therefore, for tasks in a role, `basedir` has the form
@@ -117,7 +125,7 @@ def ansible_templar(basedir: Path, templatevars: Any) -> Templar:
         basedir = basedir.parent
 
     dataloader = DataLoader()
-    dataloader.set_basedir(basedir)
+    dataloader.set_basedir(str(basedir))
     templar = Templar(dataloader, variables=templatevars)
     return templar
 
@@ -217,29 +225,26 @@ BLOCK_NAME_TO_ACTION_TYPE_MAP = {
 }
 
 
-def tokenize(line: str) -> tuple[str, list[str], dict[str, str]]:
+def tokenize(value: str) -> tuple[list[str], dict[str, str]]:
     """Parse a string task invocation."""
-    tokens = line.lstrip().split(" ")
-    if tokens[0] == "-":
-        tokens = tokens[1:]
-    if tokens[0] == "action:" or tokens[0] == "local_action:":
-        tokens = tokens[1:]
-    command = tokens[0].replace(":", "")
+    # We do not try to tokenize something very simple because it would fail to
+    # work for a case like: task_include: path with space.yml
+    if value and "=" not in value:
+        return ([value], {})
 
-    args = []
-    kwargs = {}
-    non_kv_found = False
-    for arg in tokens[1:]:
-        if "=" in arg and not non_kv_found:
-            key_value = arg.split("=", 1)
-            kwargs[key_value[0]] = key_value[1]
+    parts = split_args(value)
+    args: list[str] = []
+    kwargs: dict[str, str] = {}
+    for part in parts:
+        if "=" not in part:
+            args.append(part)
         else:
-            non_kv_found = True
-            args.append(arg)
-    return (command, args, kwargs)
+            k, v = part.split("=", 1)
+            kwargs[k] = v
+    return (args, kwargs)
 
 
-def playbook_items(pb_data: AnsibleBaseYAMLObject) -> ItemsView:  # type: ignore[type-arg]
+def playbook_items(pb_data: AnsibleBaseYAMLObject) -> ItemsView:  # type: ignore[type-arg,no-any-unimported]
     """Return a list of items from within the playbook."""
     if isinstance(pb_data, dict):
         return pb_data.items()
@@ -256,7 +261,11 @@ def set_collections_basedir(basedir: Path) -> None:
     """Set the playbook directory as playbook_paths for the collection loader."""
     # Ansible expects only absolute paths inside `playbook_paths` and will
     # produce weird errors if we use a relative one.
-    AnsibleCollectionConfig.playbook_paths = str(basedir.resolve())
+    # https://github.com/psf/black/issues/4519
+    # fmt: off
+    AnsibleCollectionConfig.playbook_paths = (  # pyright: ignore[reportAttributeAccessIssue]
+        str(basedir.resolve()))
+    # fmt: on
 
 
 def template(
@@ -294,56 +303,56 @@ class HandleChildren:
 
     def include_children(
         self,
-        basedir: str,
+        lintable: Lintable,
         k: str,
         v: Any,
         parent_type: FileType,
     ) -> list[Lintable]:
         """Include children."""
+        basedir = str(lintable.path.parent)
+
         # handle special case include_tasks: name=filename.yml
         if k in INCLUSION_ACTION_NAMES and isinstance(v, dict) and "file" in v:
             v = v["file"]
 
         # we cannot really parse any jinja2 in includes, so we ignore them
-        if not v or "{{" in v:
-            return []
-
-        if k in ("import_playbook", "ansible.builtin.import_playbook"):
-            included = Path(basedir) / v
-            if self.app.runtime.has_playbook(v, basedir=Path(basedir)):
-                if included.exists():
-                    return [Lintable(included, kind=parent_type)]
-                return []
-            msg = f"Failed to find {v} playbook."
-            logging.error(msg)
+        if not v or not isinstance(v, str) or "{{" in v:
             return []
 
         # handle include: filename.yml tags=blah
-        # pylint: disable=unused-variable
-        (command, args, kwargs) = tokenize(f"{k}: {v}")
+        (args, kwargs) = tokenize(v)
 
-        result = path_dwim(basedir, args[0])
+        if args:
+            file = args[0]
+        elif "file" in kwargs:
+            file = kwargs["file"]
+        else:
+            return []
+
+        result = path_dwim(basedir, file)
         while basedir not in ["", "/"]:
             if os.path.exists(result):
                 break
             basedir = os.path.dirname(basedir)
-            result = path_dwim(basedir, args[0])
+            result = path_dwim(basedir, file)
 
         return [Lintable(result, kind=parent_type)]
 
     def taskshandlers_children(
         self,
-        basedir: str,
+        lintable: Lintable,
         k: str,
-        v: None | Any,
+        v: Any | None,
         parent_type: FileType,
     ) -> list[Lintable]:
         """TasksHandlers Children."""
+        basedir = str(lintable.path.parent)
         results: list[Lintable] = []
         if v is None or isinstance(v, int | str):
             raise MatchError(
                 message="A malformed block was encountered while loading a block.",
                 rule=RuntimeErrorRule(),
+                lintable=lintable,
             )
         for task_handler in v:
             # ignore empty tasks, `-`
@@ -361,46 +370,32 @@ class HandleChildren:
                 continue
 
             if any(x in task_handler for x in ROLE_IMPORT_ACTION_NAMES):
-                task_handler = normalize_task_v2(task_handler)
+                task_handler = normalize_task_v2(
+                    Task(task_handler, filename=str(lintable.path)),
+                )
                 self._validate_task_handler_action_for_role(task_handler["action"])
                 name = task_handler["action"].get("name")
                 if has_jinja(name):
                     # we cannot deal with dynamic imports
                     continue
                 results.extend(
-                    self.roles_children(basedir, k, [name], parent_type),
+                    self.roles_children(lintable, k, [name], parent_type),
                 )
                 continue
 
             if "block" not in task_handler:
                 continue
 
-            results.extend(
-                self.taskshandlers_children(
-                    basedir,
-                    k,
-                    task_handler["block"],
-                    parent_type,
-                ),
-            )
-            if "rescue" in task_handler:
-                results.extend(
-                    self.taskshandlers_children(
-                        basedir,
-                        k,
-                        task_handler["rescue"],
-                        parent_type,
-                    ),
-                )
-            if "always" in task_handler:
-                results.extend(
-                    self.taskshandlers_children(
-                        basedir,
-                        k,
-                        task_handler["always"],
-                        parent_type,
-                    ),
-                )
+            for elem in ("block", "rescue", "always"):
+                if elem in task_handler:
+                    results.extend(
+                        self.taskshandlers_children(
+                            lintable,
+                            k,
+                            task_handler[elem],
+                            parent_type,
+                        ),
+                    )
 
         return results
 
@@ -408,34 +403,35 @@ class HandleChildren:
         """Verify that the task handler action is valid for role include."""
         module = th_action["__ansible_module__"]
 
+        lintable = Lintable(
+            self.rules.options.lintables[0] if self.rules.options.lintables else ".",
+        )
         if "name" not in th_action:
             raise MatchError(
                 message=f"Failed to find required 'name' key in {module!s}",
                 rule=self.rules.rules[0],
-                filename=(
-                    self.rules.options.lintables[0]
-                    if self.rules.options.lintables
-                    else "."
-                ),
+                lintable=lintable,
             )
 
         if not isinstance(th_action["name"], str):
             raise MatchError(
                 message=f"Value assigned to 'name' key on '{module!s}' is not a string.",
                 rule=self.rules.rules[1],
+                lintable=lintable,
             )
 
     def roles_children(
         self,
-        basedir: str,
+        lintable: Lintable,
         k: str,
         v: Sequence[Any],
         parent_type: FileType,
     ) -> list[Lintable]:
         """Roles children."""
         # pylint: disable=unused-argument # parent_type)
+        basedir = str(lintable.path.parent)
         results: list[Lintable] = []
-        if not v:
+        if not v or not isinstance(v, Iterable):
             # typing does not prevent junk from being passed in
             return results
         for role in v:
@@ -455,6 +451,67 @@ class HandleChildren:
                 results.extend(_look_for_role_files(basedir, role))
         return results
 
+    def import_playbook_children(
+        self,
+        lintable: Lintable,
+        k: str,  # pylint: disable=unused-argument
+        v: Any,
+        parent_type: FileType,
+    ) -> list[Lintable]:
+        """Include import_playbook children."""
+
+        def append_playbook_path(loc: str, playbook_path: list[str]) -> None:
+            possible_paths.append(
+                Path(
+                    path_dwim(
+                        os.path.expanduser(loc),
+                        os.path.join(
+                            "ansible_collections",
+                            namespace_name,
+                            collection_name,
+                            "playbooks",
+                            *playbook_path,
+                        ),
+                    ),
+                ),
+            )
+
+        # import_playbook only accepts a string as argument (no dict syntax)
+        if not isinstance(v, str):
+            return []
+
+        possible_paths = []
+        namespace_name, collection_name, *playbook_path = parse_fqcn(v)
+        if namespace_name and collection_name:
+            for loc in get_app(cached=True).runtime.config.collections_paths:
+                append_playbook_path(
+                    loc,
+                    playbook_path[:-1] + [f"{playbook_path[-1]}.yml"],
+                )
+                append_playbook_path(
+                    loc,
+                    playbook_path[:-1] + [f"{playbook_path[-1]}.yaml"],
+                )
+        else:
+            possible_paths.append(lintable.path.parent / v)
+
+        for possible_path in possible_paths:
+            if not possible_path.exists():
+                msg = f"Failed to find {v} playbook."
+            elif not self.app.runtime.has_playbook(
+                str(possible_path),
+            ):
+                msg = f"Failed to load {v} playbook due to failing syntax check."
+                break
+            elif namespace_name and collection_name:
+                # don't lint foreign playbook
+                return []
+            else:
+                return [Lintable(possible_path, kind=parent_type)]
+
+        _logger.error(msg)
+        return []
+
 
 def _get_task_handler_children_for_tasks_or_playbooks(
     task_handler: dict[str, Any],
@@ -472,10 +529,24 @@ def _get_task_handler_children_for_tasks_or_playbooks(
             if not task_handler or isinstance(task_handler, str):  # pragma: no branch
                 continue
 
-            file_name = task_handler[task_handler_key]
-            if isinstance(file_name, Mapping) and file_name.get("file", None):
-                file_name = file_name["file"]
+            file_name = ""
+            action_args = task_handler[task_handler_key]
+            if isinstance(action_args, str):
+                (args, kwargs) = tokenize(action_args)
+                if len(args) == 1:
+                    file_name = args[0]
+                elif kwargs.get("file", None):
+                    file_name = kwargs["file"]
+                else:
+                    # ignore invalid data (syntax check will outside the scope)
+                    continue
 
+            if isinstance(action_args, Mapping) and action_args.get("file", None):
+                file_name = action_args["file"]
+
+            if not file_name:
+                # ignore invalid data (syntax check will outside the scope)
+                continue
             f = path_dwim(basedir, file_name)
             while basedir not in ["", "/"]:
                 if os.path.exists(f):
@@ -489,21 +560,38 @@ def _get_task_handler_children_for_tasks_or_playbooks(
 
 def _rolepath(basedir: str, role: str) -> str | None:
     role_path = None
+    namespace_name, collection_name, role_name = parse_fqcn(role)
 
     possible_paths = [
         # if included from a playbook
-        path_dwim(basedir, os.path.join("roles", role)),
-        path_dwim(basedir, role),
+        path_dwim(basedir, os.path.join("roles", role_name)),
+        path_dwim(basedir, role_name),
         # if included from roles/[role]/meta/main.yml
-        path_dwim(basedir, os.path.join("..", "..", "..", "roles", role)),
-        path_dwim(basedir, os.path.join("..", "..", role)),
+        path_dwim(basedir, os.path.join("..", "..", "..", "roles", role_name)),
+        path_dwim(basedir, os.path.join("..", "..", role_name)),
         # if checking a role in the current directory
-        path_dwim(basedir, os.path.join("..", role)),
+        path_dwim(basedir, os.path.join("..", role_name)),
     ]
 
     for loc in get_app(cached=True).runtime.config.default_roles_path:
         loc = os.path.expanduser(loc)
-        possible_paths.append(path_dwim(loc, role))
+        possible_paths.append(path_dwim(loc, role_name))
+
+    if namespace_name and collection_name:
+        for loc in get_app(cached=True).runtime.config.collections_paths:
+            loc = os.path.expanduser(loc)
+            possible_paths.append(
+                path_dwim(
+                    loc,
+                    os.path.join(
+                        "ansible_collections",
+                        namespace_name,
+                        collection_name,
+                        "roles",
+                        role_name,
+                    ),
+                ),
+            )
 
     possible_paths.append(path_dwim(basedir, ""))
 
@@ -566,13 +654,14 @@ def _extract_ansible_parsed_keys_from_task(
     return result
 
 
-def normalize_task_v2(task: dict[str, Any]) -> dict[str, Any]:
+def normalize_task_v2(task: Task) -> dict[str, Any]:
     """Ensure tasks have a normalized action key and strings are converted to python objects."""
+    raw_task = task.raw_task
     result: dict[str, Any] = {}
     ansible_parsed_keys = ("action", "local_action", "args", "delegate_to")
 
-    if is_nested_task(task):
-        _extract_ansible_parsed_keys_from_task(result, task, ansible_parsed_keys)
+    if is_nested_task(raw_task):
+        _extract_ansible_parsed_keys_from_task(result, raw_task, ansible_parsed_keys)
         # Add dummy action for block/always/rescue statements
         result["action"] = {
             "__ansible_module__": "block/always/rescue",
@@ -581,7 +670,7 @@ def normalize_task_v2(task: dict[str, Any]) -> dict[str, Any]:
 
         return result
 
-    sanitized_task = _sanitize_task(task)
+    sanitized_task = _sanitize_task(raw_task)
     mod_arg_parser = ModuleArgsParser(sanitized_task)
 
     try:
@@ -592,8 +681,8 @@ def normalize_task_v2(task: dict[str, Any]) -> dict[str, Any]:
         raise MatchError(
             rule=AnsibleParserErrorRule(),
             message=exc.message,
-            filename=task.get(FILENAME_KEY, "Unknown"),
-            lineno=task.get(LINE_NUMBER_KEY, 0),
+            lintable=Lintable(task.filename or ""),
+            lineno=raw_task.get(LINE_NUMBER_KEY, 1),
         ) from exc
 
     # denormalize shell -> command conversion
@@ -603,7 +692,7 @@ def normalize_task_v2(task: dict[str, Any]) -> dict[str, Any]:
 
     _extract_ansible_parsed_keys_from_task(
         result,
-        task,
+        raw_task,
         (*ansible_parsed_keys, action),
     )
 
@@ -625,17 +714,6 @@ def normalize_task_v2(task: dict[str, Any]) -> dict[str, Any]:
     return result
 
 
-def normalize_task(task: dict[str, Any], filename: str) -> dict[str, Any]:
-    """Unify task-like object structures."""
-    ansible_action_type = task.get("__ansible_action_type__", "task")
-    if "__ansible_action_type__" in task:
-        del task["__ansible_action_type__"]
-    task = normalize_task_v2(task)
-    task[FILENAME_KEY] = filename
-    task["__ansible_action_type__"] = ansible_action_type
-    return task
-
-
 def task_to_str(task: dict[str, Any]) -> str:
     """Make a string identifier for the given task."""
     name = task.get("name")
@@ -667,7 +745,8 @@ def task_to_str(task: dict[str, Any]) -> str:
     return f"{action['__ansible_module__']} {' '.join(args)}"
 
 
-def extract_from_list(
+# pylint: disable=too-many-nested-blocks
+def extract_from_list(  # type: ignore[no-any-unimported]
     blocks: AnsibleBaseYAMLObject,
     candidates: list[str],
     *,
@@ -675,23 +754,24 @@ def extract_from_list(
 ) -> list[Any]:
     """Get action tasks from block structures."""
     results = []
-    for block in blocks:
-        for candidate in candidates:
-            if isinstance(block, dict) and candidate in block:
-                if isinstance(block[candidate], list):
-                    subresults = add_action_type(block[candidate], candidate)
-                    if recursive:
-                        subresults.extend(
-                            extract_from_list(
-                                subresults,
-                                candidates,
-                                recursive=recursive,
-                            ),
-                        )
-                    results.extend(subresults)
-                elif block[candidate] is not None:
-                    msg = f"Key '{candidate}' defined, but bad value: '{block[candidate]!s}'"
-                    raise RuntimeError(msg)
+    if isinstance(blocks, Iterable):
+        for block in blocks:
+            for candidate in candidates:
+                if isinstance(block, dict) and candidate in block:
+                    if isinstance(block[candidate], list):
+                        subresults = add_action_type(block[candidate], candidate)
+                        if recursive:
+                            subresults.extend(
+                                extract_from_list(
+                                    subresults,
+                                    candidates,
+                                    recursive=recursive,
+                                ),
+                            )
+                        results.extend(subresults)
+                    elif block[candidate] is not None:
+                        msg = f"Key '{candidate}' defined, but bad value: '{block[candidate]!s}'"
+                        raise RuntimeError(msg)
     return results
 
 
@@ -748,10 +828,11 @@ class Task(dict[str, Any]):
         """
         if "args" in self.raw_task:
             return self.raw_task["args"]
-        result = {}
-        for k, v in self.normalized_task["action"].items():
-            if k not in ANNOTATION_KEYS:
-                result[k] = v
+        result = {
+            k: v
+            for k, v in self.normalized_task["action"].items()
+            if k not in ANNOTATION_KEYS
+        }
         return result
 
     @property
@@ -759,10 +840,7 @@ class Task(dict[str, Any]):
         """Return the name of the task."""
         if not hasattr(self, "_normalized_task"):
             try:
-                self._normalized_task = normalize_task(
-                    self.raw_task,
-                    filename=self.filename,
-                )
+                self._normalized_task = self._normalize_task()
             except MatchError as err:
                 self.error = err
                 # When we cannot normalize it, we just use the raw task instead
@@ -773,6 +851,16 @@ class Task(dict[str, Any]):
             raise TypeError(msg)
         return self._normalized_task
 
+    def _normalize_task(self) -> dict[str, Any]:
+        """Unify task-like object structures."""
+        ansible_action_type = self.raw_task.get("__ansible_action_type__", "task")
+        if "__ansible_action_type__" in self.raw_task:
+            del self.raw_task["__ansible_action_type__"]
+        task = normalize_task_v2(self)
+        task[FILENAME_KEY] = self.filename
+        task["__ansible_action_type__"] = ansible_action_type
+        return task
+
     @property
     def skip_tags(self) -> list[str]:
         """Return the list of tags to skip."""
@@ -785,8 +873,9 @@ class Task(dict[str, Any]):
         if isinstance(self._normalized_task, dict):
             file_name = str(self._normalized_task["action"].get(FILENAME_KEY, None))
             if file_name:
-                is_handler_file = "handlers" in str(file_name)
-        return is_handler_file if is_handler_file else ".handlers[" in self.position
+                paths = file_name.split("/")
+                is_handler_file = "handlers" in paths
+        return is_handler_file or ".handlers[" in self.position
 
     def __repr__(self) -> str:
         """Return a string representation of the task."""
@@ -804,8 +893,33 @@ class Task(dict[str, Any]):
         """Provide support for 'key in task'."""
         yield from (f for f in self.normalized_task)
 
+    def get_error_line(self, path: list[str | int]) -> int:
+        """Return error line number."""
+        ctx = self.normalized_task
+        line = self.normalized_task[LINE_NUMBER_KEY]
+        for _ in path:
+            if (
+                isinstance(ctx, collections.abc.Container) and _ in ctx
+            ):  # isinstance(ctx, collections.abc.Container) and
+                value = ctx.get(  # pyright: ignore[reportAttributeAccessIssue]
+                    _  # pyright: ignore[reportArgumentType]
+                )
+                if isinstance(value, dict):
+                    ctx = value
+                if (
+                    isinstance(ctx, collections.abc.Container)
+                    and LINE_NUMBER_KEY in ctx
+                ):
+                    line = ctx[LINE_NUMBER_KEY]  # pyright: ignore[reportIndexIssue]
+                # else:
+                #     break
+        if not isinstance(line, int):
+            msg = "Line number is not an integer"
+            raise TypeError(msg)
+        return line
 
-def task_in_list(
+
+def task_in_list(  # type: ignore[no-any-unimported]
     data: AnsibleBaseYAMLObject,
     file: Lintable,
     kind: str,
@@ -813,8 +927,9 @@ def task_in_list(
 ) -> Iterator[Task]:
     """Get action tasks from block structures."""
 
-    def each_entry(data: AnsibleBaseYAMLObject, position: str) -> Iterator[Task]:
-        if not data:
+    def each_entry(data: AnsibleBaseYAMLObject, position: str) -> Iterator[Task]:  # type: ignore[no-any-unimported]
+
+        if not data or not isinstance(data, Iterable):
             return
         for entry_index, entry in enumerate(data):
             if not entry:
@@ -854,43 +969,35 @@ def task_in_list(
         yield from each_entry(data, position)
 
 
-def add_action_type(actions: AnsibleBaseYAMLObject, action_type: str) -> list[Any]:
+def add_action_type(  # type: ignore[no-any-unimported]
+    actions: AnsibleBaseYAMLObject, action_type: str
+) -> AnsibleSequence:
     """Add action markers to task objects."""
-    results = []
-    for action in actions:
-        # ignore empty task
-        if not action:
-            continue
-        action["__ansible_action_type__"] = BLOCK_NAME_TO_ACTION_TYPE_MAP[action_type]
-        results.append(action)
+    results = AnsibleSequence()
+    if isinstance(actions, Iterable):
+        for action in actions:
+            # ignore empty task
+            if not action:
+                continue
+            action["__ansible_action_type__"] = BLOCK_NAME_TO_ACTION_TYPE_MAP[
+                action_type
+            ]
+            results.append(action)
     return results
 
 
-def get_action_tasks(data: AnsibleBaseYAMLObject, file: Lintable) -> list[Any]:
-    """Get a flattened list of action tasks from the file."""
-    tasks = []
-    if file.kind in ["tasks", "handlers"]:
-        tasks = add_action_type(data, file.kind)
-    else:
-        tasks.extend(extract_from_list(data, PLAYBOOK_TASK_KEYWORDS))
-
-    # Add sub-elements of block/rescue/always to tasks list
-    tasks.extend(extract_from_list(tasks, NESTED_TASK_KEYS, recursive=True))
-
-    return tasks
-
-
 @cache
-def parse_yaml_linenumbers(
+def parse_yaml_linenumbers(  # type: ignore[no-any-unimported]
     lintable: Lintable,
-) -> AnsibleBaseYAMLObject:
+) -> AnsibleBaseYAMLObject | None:
     """Parse yaml as ansible.utils.parse_yaml but with linenumbers.
 
     The line numbers are stored in each node's LINE_NUMBER_KEY key.
     """
-    result = []
+    result = AnsibleSequence()
 
-    def compose_node(parent: yaml.nodes.Node, index: int) -> yaml.nodes.Node:
+    # signature of Composer.compose_node
+    def compose_node(parent: yaml.nodes.Node | None, index: int) -> yaml.nodes.Node:
         # the line number where the previous token has ended (plus empty lines)
         line = loader.line
         node = Composer.compose_node(loader, parent, index)
@@ -900,14 +1007,15 @@ def parse_yaml_linenumbers(
         node.__line__ = line + 1  # type: ignore[attr-defined]
         return node
 
-    def construct_mapping(
-        node: AnsibleBaseYAMLObject,
-        *,
-        deep: bool = False,
+    # signature of AnsibleConstructor.construct_mapping
+    def construct_mapping(  # type: ignore[no-any-unimported]
+        node: yaml.MappingNode,
+        deep: bool = False,  # noqa: FBT002
     ) -> AnsibleMapping:
+        # pyright: ignore[reportArgumentType]
         mapping = AnsibleConstructor.construct_mapping(loader, node, deep=deep)
-        if hasattr(node, "__line__"):
-            mapping[LINE_NUMBER_KEY] = node.__line__
+        if hasattr(node, LINE_NUMBER_KEY):
+            mapping[LINE_NUMBER_KEY] = getattr(node, LINE_NUMBER_KEY)
         else:
             mapping[LINE_NUMBER_KEY] = mapping._line_number  # noqa: SLF001
         mapping[FILENAME_KEY] = lintable.path
@@ -918,7 +1026,9 @@ def parse_yaml_linenumbers(
         if "vault_password" in inspect.getfullargspec(AnsibleLoader.__init__).args:
             kwargs["vault_password"] = DEFAULT_VAULT_PASSWORD
         loader = AnsibleLoader(lintable.content, **kwargs)
+        # redefine Composer.compose_node
         loader.compose_node = compose_node
+        # redefine AnsibleConstructor.construct_mapping
         loader.construct_mapping = construct_mapping
         # while Ansible only accepts single documents, we also need to load
         # multi-documents, as we attempt to load any YAML file, not only
@@ -929,8 +1039,8 @@ def parse_yaml_linenumbers(
                 break
             result.append(data)
     except (
-        yaml.parser.ParserError,
-        yaml.scanner.ScannerError,
+        ParserError,
+        ScannerError,
         yaml.constructor.ConstructorError,
         ruamel.yaml.parser.ParserError,
     ) as exc:
@@ -1080,7 +1190,7 @@ def parse_examples_from_plugin(lintable: Lintable) -> tuple[int, str]:
 
 
 @lru_cache
-def load_plugin(name: str) -> PluginLoadContext:
+def load_plugin(name: str) -> PluginLoadContext:  # type: ignore[no-any-unimported]
     """Return loaded ansible plugin/module."""
     loaded_module = action_loader.find_plugin_with_context(
         name,
@@ -1101,3 +1211,8 @@ def load_plugin(name: str) -> PluginLoadContext:
             check_aliases=True,
         )
     return loaded_module
+
+
+def parse_fqcn(name: str) -> tuple[str, ...]:
+    """Parse name parameter into FQCN segments."""
+    return tuple(name.split(".")) if is_fqcn(name) else ("", "", name)
